function [moist_ape, p_ref]=moist_ape(T, p, r, do_nonconvective, lat_parcel, lat_grid, thermo_type)

 % Divide and conquer algorithm for moist available potential energy

 % The divide and conquer algorithm was introduced in Stansifer, O'Gorman and Holt, QJRMS, 143, 288-292, 2017. The python code used for that paper is available at https://github.com/estansifer/mape/. The version given here differs by using  different thermodynamic constants and a different treatment of moisture (either 'full' which includes ice or 'simple' which has constant latent heat of vaporization)

 % The input profiles are assumed to be 1d, with evenly spaced pressure levels, and no liquid water or ice. An example of how to apply this function to a zonal and time mean state that varies with latitude is given in moist_mape_ipcc.m

 % The input arrays lat_parcel and lat_grid are only needed if do_nonconvective = 1 (the "non-convective" moist ape discussed in O'Gorman, PNAS, 107, 19176-19180, 2010 that does not allow the reordering of the pressure of air parcels originating at a given latitude)

 % Variable key:
 % moist_ape moist available potential energy (or gcape) in J/kg
 % p_ref reference pressure (Pa)
 % T temperature (K)
 % p pressure (Pa)
 % r generalized relative humidity (ratio of vapor pressures when below one,
 % but a ratio of mixing ratios when above one) (note fraction rather than percent)
 % s entropy
 % h enthalpy
 % w mixing ratio
 % L latent heat of evaporation or sublimation
 % e vapor pressure
 % wt total mixing ratio
 % lat_parcel latitudes of parcels in the profile
 % lat_grid the grid of possible latitudes


 error(nargchk(3, 7, nargin))

 % thermo_type: default is full rather than simple
 if nargin < 7
   thermo_type = 'full';
 end

 if nargin<4
  do_nonconvective = 0; % default is full moist ape (allows for convection)
 end

 if do_nonconvective&nargin<6
  error('must supply latitudes if do_nonconvective=1')
 end

 if p(2)<p(1)
  error('pressure ordering of levels not what expected')
 end

 T = T(:);
 p = p(:);
 r = r(:);

 % find basic thermodynamic quantities
 if all(r==0)
  disp('dry ape')
  e         = zeros(size(T));
  Tc        = ones(size(T))*NaN;
  pc        = ones(size(T))*NaN;
  wt        = zeros(size(T));
  dh_to_sat = zeros(size(T));
 else
  [e, Tc, pc, wt, dh_to_sat] = moist_ape_thermo(T, p, r, thermo_type);
 end

 % index(k): the index of the parcel that ends up at k 
 index = subsolve(1,length(p),[1:length(p)],T, p, r, e, Tc, pc, wt, dh_to_sat, thermo_type, do_nonconvective, lat_grid, lat_parcel);

 moist_ape = 0;
 for k=1:length(p)
  i           = index(k);
  p_ref(i)    = p(k); % the reference pressure each parcel moves to

  [dummy, dh] = moist_ape_parcel_move(T(i), p(i), r(i), e(i), Tc(i), pc(i), wt(i), dh_to_sat(i), p(k), thermo_type);
  moist_ape       = moist_ape - dh; % parcels have same mass (dp constant) so no
                            % (1+w) weighting needed
  dh_all(i)       = dh;
 end


 % result in J/kg
 moist_ape = moist_ape/length(p);

end

function index_reordered = subsolve(kmin, kmax, index, T, p, r, e, Tc, pc, wt, dh_to_sat, thermo_type, do_nonconvective, lat_grid, lat_parcel)

 % reorder domain from kmin to kmax, then recursively reorder top and bottom halves

 n = kmax-kmin+1; 

 if n==1
  index_reordered = index; % end of recursion
 else

  % find mid pressure
  m = floor((n+1)/2)+kmin-1;
  p_mid = p(m);
  
  i = index(kmin:kmax);
  
  % sort by virtual temperature at mid pressure 
  Tvf_mid = moist_ape_parcel_move(T(i), p(i), r(i), e(i), Tc(i), pc(i), wt(i), dh_to_sat(i), p_mid, thermo_type);
  
  Tvf_mid = Tvf_mid(:);


  if do_nonconvective
   % don't allow reordering of pressures of parcels in each column of the untransformed state (parcels can move wrt latitude though)

   index_reordered = index;
   for k=kmin:kmax
    % only select from set of highest available parcels in each column
    Tvf_mid_avail = Tvf_mid*NaN; % NaN marks non-available parcels
    for j=1:length(lat_grid)
     available = find(lat_parcel(index(kmin:kmax))==lat_grid(j)&isfinite(Tvf_mid)');
     if ~isempty(available)
      highest = min(available);
      Tvf_mid_avail(highest) = Tvf_mid(highest);
     end

    end
    [dummy, s] = max(Tvf_mid_avail);
    Tvf_mid(s) = NaN; % parcel no longer available
    index_reordered(k) = index(s+kmin-1);
   end

  else
   [dummy, s] = sort(Tvf_mid,1,'descend');
   index_reordered = index;
   index_reordered(kmin:kmax) = index(s+kmin-1);
  end

  index_top_reordered = subsolve(kmin, m, index_reordered, T, p, r, e, Tc, pc, wt, dh_to_sat, thermo_type, do_nonconvective, lat_grid, lat_parcel);
  index_reordered = index_top_reordered;

  index_bot_reordered = subsolve(m+1, kmax, index_reordered, T, p, r, e, Tc, pc, wt, dh_to_sat, thermo_type, do_nonconvective, lat_grid, lat_parcel);
  index_reordered = index_bot_reordered;
  
 end
  
end

